"""Module containing account related models."""

# vim: set tw=80 ts=4 sw=4 sts=4:

from . import db

from .operations import Operation


# pylint: disable=no-member,invalid-name,too-few-public-methods,no-self-use
class Account(db.Model):
    """Model class to handle accounts."""
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(200), nullable=False)
    authorized_overdraft = db.Column(db.Integer, nullable=True, default=0)

    def __init__(self, name, authorized_overdraft):
        self.name = name
        self.authorized_overdraft = authorized_overdraft

    @classmethod
    def query(cls):
        """Return a query for this class method."""
        return db.session.query(
            cls
        ).order_by(
            cls.name
        )

    def balances(self):
        """Return the balances of this account."""
        return db.session.query(
            db.func.coalesce(
                db.func.sum(
                    Operation.value
                ), 0
            ).label("future"),
            db.func.coalesce(
                db.func.sum(
                    Operation.value
                ).filter(
                    Operation.pointed
                ), 0
            ).label("pointed"),
            db.func.coalesce(
                db.func.sum(
                    Operation.value,
                ).filter(
                    Operation.operation_date <= db.func.current_date()
                ), 0
            ).label("current"),
        ).filter(
            Operation.account_id == self.id
        ).one()

    def income(self, begin, end):
        """Return the income for a specific time period."""
        query = db.session.query(
            db.func.coalesce(
                db.func.sum(
                    Operation.value
                ).filter(
                    db.func.sign(Operation.value) == -1
                ), 0
            ).label("expenses"),
            db.func.coalesce(
                db.func.sum(
                    Operation.value
                ).filter(
                    db.func.sign(Operation.value) == 1
                ), 0
            ).label("revenues"),
            db.func.coalesce(
                db.func.sum(Operation.value), 0
            ).label("income")
        ).filter(
            Operation.account_id == self.id,
        )

        if begin:
            query = query.filter(Operation.operation_date >= str(begin))

        if end:
            query = query.filter(Operation.operation_date <= str(end))

        return query.one()

    @db.validates('authorized_overdraft')
    def validate_authorized_overdraft(self, key, authorized_overdraft):
        """Validator for authorized_overdraft : must be negative."""
        del key

        assert authorized_overdraft <= 0

        return authorized_overdraft
